/*
 * Dynamic loading of modules into the kernel.
 */

#ifndef _LINUX_MODULE_H
#define _LINUX_MODULE_H

#include <linux/unistd.h>

/* values of module.state */
#define MOD_UNINITIALIZED 0
#define MOD_RUNNING 1
#define MOD_DELETED 2

/* maximum length of module name */
#define MOD_MAX_NAME 64

/* maximum length of symbol name */
#define SYM_MAX_NAME 60


struct module {
	struct module *next;
	char *name;
	int size;			/* size of module in pages */
	void* addr;			/* address of module */
	int state;
	void (*cleanup)(void);		/* cleanup routine */
};


struct mod_routines {
	int (*init)(void);		/* initialization routine */
	void (*cleanup)(void);		/* cleanup routine */
};


struct kernel_sym {
	unsigned long value;		/* value of symbol */
	char name[SYM_MAX_NAME];	/* name of symbol */
};


/*
 * The first word of the module contains the use count.
 */
#define GET_USE_COUNT(module)	(* (int *) (module)->addr)

extern struct module *module_list;

/*
 * This is probably not the right place nor the right way to create these
 * interfaces. If somebody knows how to do this correctly let him do so !
 */
_syscall1( int, delete_module, const char*, name)
_syscall1( int, get_kernel_syms, struct kernel_sym*, buffer)
_syscall2( int, create_module, const char*, name, unsigned long, size)
_syscall4( int, init_module, const char*, name, void*, code, 
          unsigned, codesize, struct mod_routines*, routines);

#endif
